Mirror of the gdb-patches mailing list
 help / color / mirror / Atom feed
From: Tom de Vries via Gdb-patches <gdb-patches@sourceware.org>
To: gdb-patches@sourceware.org
Subject: Re: [PATCH][gdb/testsuite] Reimplement complaints selftest as unittest
Date: Fri, 3 Sep 2021 13:18:57 +0200	[thread overview]
Message-ID: <b29f5118-1131-e2a6-1384-9ac0451f4643@suse.de> (raw)
In-Reply-To: <20210902134249.GA25158@delia>

[-- Attachment #1: Type: text/plain, Size: 422 bytes --]

On 9/2/21 3:42 PM, Tom de Vries wrote:
> Something specific to the new unittest is that we check the output as well.
> To this end, we add SELFTEST_OUTPUT and SELFTEST_EXPECTED_OUTPUT markers:

I tried it in another way: capturing the output, which means there's no
need for the testsuite to verify the output.

Passes build & gdb.gdb/unittest.exp, currently doing full regression
test run.

Any comments?

Thanks,
- Tom


[-- Attachment #2: 0002-gdb-testsuite-Reimplement-gdb.gdb-complaints.exp-as-unittest.patch --]
[-- Type: text/x-patch, Size: 10451 bytes --]

[gdb/testsuite] Reimplement gdb.gdb/complaints.exp as unittest

When building gdb with "-Wall -O2 -g -flto=auto", I run into:
...
(gdb) call clear_complaints()^M
No symbol "clear_complaints" in current context.^M
(gdb) FAIL: gdb.gdb/complaints.exp: clear complaints
...

The problem is that lto has optimized away the clear_complaints function
and consequently the selftest doesn't work.

Fix this by reimplementing the selftest as a unit test.

Factor out two new functions:
- void
  execute_fn_to_ui_file (struct ui_file *file, std::function<void(void)> fn);
- std::string
  execute_fn_to_string (std::function<void(void)> fn, bool term_out);
and use the latter to capture the complaints output.

Tested on x86_64-linux.

---
 gdb/complaints.c                     |  52 ++++++++++++++++
 gdb/gdbcmd.h                         |  26 +++++---
 gdb/testsuite/gdb.gdb/complaints.exp | 117 -----------------------------------
 gdb/top.c                            |  31 +++++++---
 4 files changed, 94 insertions(+), 132 deletions(-)

diff --git a/gdb/complaints.c b/gdb/complaints.c
index fecbcd74ce5..525a3a7eacf 100644
--- a/gdb/complaints.c
+++ b/gdb/complaints.c
@@ -21,6 +21,7 @@
 #include "complaints.h"
 #include "command.h"
 #include "gdbcmd.h"
+#include "gdbsupport/selftest.h"
 #include <unordered_map>
 
 /* Map format strings to counters.  */
@@ -74,6 +75,53 @@ complaints_show_value (struct ui_file *file, int from_tty,
 		    value);
 }
 
+#if GDB_SELF_TEST
+namespace selftests {
+
+/* Entry point for complaints unit tests.  */
+
+static void
+test_complaints ()
+{
+  std::unordered_map<const char *, int> tmp;
+  scoped_restore reset_counters = make_scoped_restore (&counters, tmp);
+  scoped_restore reset_stop_whining = make_scoped_restore (&stop_whining, 2);
+
+#define CHECK_COMPLAINT(STR, CNT)					\
+  do									\
+    {									\
+      std::string output;						\
+      output = execute_fn_to_string ([]() { complaint (STR); }, false);	\
+      std::string expected						\
+	= _("During symbol reading: ") + std::string (STR "\n");	\
+      SELF_CHECK (output == expected);					\
+      SELF_CHECK (counters[STR] == CNT);				\
+    } while (0)
+
+#define CHECK_COMPLAINT_SILENT(STR, CNT)				\
+  do									\
+    {									\
+      std::string output;						\
+      output = execute_fn_to_string ([]() { complaint (STR); }, false);	\
+      SELF_CHECK (output.empty ());					\
+      SELF_CHECK (counters[STR] == CNT);				\
+    } while (0)
+
+  CHECK_COMPLAINT ("maintenance complaint 0", 1);
+  CHECK_COMPLAINT ("maintenance complaint 0", 2);
+  CHECK_COMPLAINT_SILENT ("maintenance complaint 0", 3);
+  CHECK_COMPLAINT ("maintenance complaint 1", 1);
+  clear_complaints ();
+  CHECK_COMPLAINT ("maintenance complaint 0", 1);
+
+#undef CHECK_COMPLAINT
+#undef CHECK_COMPLAINT_SILENT
+}
+
+
+} // namespace selftests
+#endif /* GDB_SELF_TEST */
+
 void _initialize_complaints ();
 void
 _initialize_complaints ()
@@ -84,4 +132,8 @@ Set max number of complaints about incorrect symbols."), _("\
 Show max number of complaints about incorrect symbols."), NULL,
 			    NULL, complaints_show_value,
 			    &setlist, &showlist);
+
+#if GDB_SELF_TEST
+  selftests::register_test ("complaints", selftests::test_complaints);
+#endif /* GDB_SELF_TEST */
 }
diff --git a/gdb/testsuite/gdb.gdb/complaints.exp b/gdb/testsuite/gdb.gdb/complaints.exp
deleted file mode 100644
index c70825b6623..00000000000
--- a/gdb/testsuite/gdb.gdb/complaints.exp
+++ /dev/null
@@ -1,117 +0,0 @@
-# Copyright 2002-2021 Free Software Foundation, Inc.
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-# This file was written by Andrew Cagney (cagney at redhat dot com),
-# derived from xfullpath.exp (written by Joel Brobecker), derived from
-# selftest.exp (written by Rob Savoye).
-
-load_lib selftest-support.exp
-
-if [target_info exists gdb,noinferiorio] {
-    verbose "Skipping because of no inferiorio capabilities."
-    return
-}
-
-# Similar to gdb_test_stdio, except no \r\n is expected before
-# $gdb_prompt in the $gdb_spawn_id.
-
-proc test_complaint {test inferior_io_re msg} {
-    global inferior_spawn_id gdb_spawn_id
-    global gdb_prompt
-
-    set inferior_matched 0
-    set gdb_matched 0
-
-    gdb_test_multiple $test $msg {
-	-i $inferior_spawn_id -re "$inferior_io_re" {
-	    set inferior_matched 1
-	    if {!$gdb_matched} {
-		exp_continue
-	    }
-	}
-	-i $gdb_spawn_id -re "$gdb_prompt $" {
-	    set gdb_matched 1
-	    if {!$inferior_matched} {
-		exp_continue
-	    }
-	}
-    }
-
-    verbose -log "inferior_matched=$inferior_matched, gdb_matched=$gdb_matched"
-    gdb_assert {$inferior_matched && $gdb_matched} $msg
-}
-
-proc test_initial_complaints { } {
-    # Unsupress complaints
-    gdb_test "set stop_whining = 2"
-
-    gdb_test_no_output "set var \$cstr = \"Register a complaint\""
-
-    # Prime the system
-    gdb_test_stdio \
-	"call complaint_internal (\$cstr)" \
-	"During symbol reading: Register a complaint"
-
-    # Re-issue the first message #1
-    with_test_prefix "re-issue" {
-	gdb_test_stdio \
-	    "call complaint_internal (\$cstr)" \
-	    "During symbol reading: Register a complaint"
-    }
-
-    # Add a second complaint, expect it
-    gdb_test_stdio \
-	"call complaint_internal (\"Testing! Testing! Testing!\")" \
-	"During symbol reading: Testing. Testing. Testing."
-
-    return 0
-}
-
-# Check that nothing comes out when there haven't been any real
-# complaints.  Note that each test is really checking the previous
-# command.
-
-proc test_empty_complaint { cmd msg } {
-    global gdb_prompt
-    global inferior_spawn_id gdb_spawn_id
-
-    if {$gdb_spawn_id == $inferior_spawn_id} {
-	gdb_test_no_output $cmd $msg
-    } else {
-	set seen_output 0
-	gdb_test_multiple $cmd $msg {
-	    -i $inferior_spawn_id -re "." {
-		set seen_output 1
-		exp_continue
-	    }
-	    -i $gdb_spawn_id "$gdb_prompt $" {
-		gdb_assert !$seen_output $msg
-	    }
-	}
-    }
-}
-
-proc test_empty_complaints { } {
-
-    test_empty_complaint "call clear_complaints()" \
-	    "clear complaints"
-
-    return 0
-}
-
-do_self_tests captured_command_loop {
-    test_initial_complaints
-    test_empty_complaints
-}
diff --git a/gdb/top.c b/gdb/top.c
index 0c49f4f9eb4..7e95ed3969c 100644
--- a/gdb/top.c
+++ b/gdb/top.c
@@ -696,12 +696,10 @@ execute_command (const char *p, int from_tty)
   cleanup_if_error.release ();
 }
 
-/* Run execute_command for P and FROM_TTY.  Sends its output to FILE,
-   do not display it to the screen.  BATCH_FLAG will be
-   temporarily set to true.  */
+/* See gdbcmd.h.  */
 
 void
-execute_command_to_ui_file (struct ui_file *file, const char *p, int from_tty)
+execute_fn_to_ui_file (struct ui_file *file, std::function<void(void)> fn)
 {
   /* GDB_STDOUT should be better already restored during these
      restoration callbacks.  */
@@ -724,22 +722,39 @@ execute_command_to_ui_file (struct ui_file *file, const char *p, int from_tty)
     scoped_restore save_stdtargerr
       = make_scoped_restore (&gdb_stdtargerr, file);
 
-    execute_command (p, from_tty);
+    fn ();
   }
 }
 
 /* See gdbcmd.h.  */
 
 std::string
-execute_command_to_string (const char *p, int from_tty,
-			   bool term_out)
+execute_fn_to_string (std::function<void(void)> fn, bool term_out)
 {
   string_file str_file (term_out);
 
-  execute_command_to_ui_file (&str_file, p, from_tty);
+  execute_fn_to_ui_file (&str_file, fn);
   return std::move (str_file.string ());
 }
 
+/* See gdbcmd.h.  */
+
+void
+execute_command_to_ui_file (struct ui_file *file,
+			    const char *p, int from_tty)
+{
+  execute_fn_to_ui_file (file, [=]() { execute_command (p, from_tty); });
+}
+
+/* See gdbcmd.h.  */
+
+std::string
+execute_command_to_string (const char *p, int from_tty,
+			   bool term_out)
+{
+  return
+    execute_fn_to_string ([=]() { execute_command (p, from_tty); }, term_out);
+}
 \f
 /* When nonzero, cause dont_repeat to do nothing.  This should only be
    set via prevent_dont_repeat.  */

  reply	other threads:[~2021-09-03 11:19 UTC|newest]

Thread overview: 3+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2021-09-02 13:42 Tom de Vries via Gdb-patches
2021-09-03 11:18 ` Tom de Vries via Gdb-patches [this message]
2021-09-08 19:02   ` Tom Tromey

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=b29f5118-1131-e2a6-1384-9ac0451f4643@suse.de \
    --to=gdb-patches@sourceware.org \
    --cc=tdevries@suse.de \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
diff --git a/gdb/gdbcmd.h b/gdb/gdbcmd.h
index f541ebd92d2..27550c1faee 100644
--- a/gdb/gdbcmd.h
+++ b/gdb/gdbcmd.h
@@ -134,17 +134,29 @@ extern struct cmd_list_element *save_cmdlist;
 
 extern void execute_command (const char *, int);
 
-/* Run execute_command for P and FROM_TTY.  Capture its output into the
-   returned string, do not display it to the screen.  The global BATCH_FLAG
-   will temporarily be set to true.  When TERM_OUT is true the output is
-   collected with terminal behaviour (e.g. with styling).  When TERM_OUT is
-   false raw output will be collected (e.g. no styling).  */
+/* Run FN.  Sends its output to FILE, do not display it to the screen.
+   The global BATCH_FLAG will be temporarily set to true.  */
+
+extern void execute_fn_to_ui_file (struct ui_file *file, std::function<void(void)> fn);
+
+/* Run FN.  Capture its output into the returned string, do not display it
+   to the screen.  The global BATCH_FLAG will temporarily be set to true.
+   When TERM_OUT is true the output is collected with terminal behaviour
+   (e.g. with styling).  When TERM_OUT is false raw output will be collected
+   (e.g. no styling).  */
+
+extern std::string execute_fn_to_string (std::function<void(void)> fn, bool term_out);
+
+/* As execute_fn_to_ui_file, but run execute_command for P and FROM_TTY.  */
 
-extern std::string execute_command_to_string (const char *p, int from_tty,
-					      bool term_out);
 extern void execute_command_to_ui_file (struct ui_file *file,
 					const char *p, int from_tty);
 
+/* As execute_fn_to_string, but run execute_command for P and FROM_TTY.  */
+
+extern std::string execute_command_to_string (const char *p, int from_tty,
+					      bool term_out);
+
 extern void print_command_line (struct command_line *, unsigned int,
 				struct ui_file *);
 extern void print_command_lines (struct ui_out *,